@@ -39,8 +39,11 @@ const char usage[]= "usage: id3 -[tTaAycg] `text' file1 [file2...]\n\
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include "genre.h"
 
+
 struct id3 {
 	char tag[3];
 	char title[30];
@@ -55,12 +58,14 @@ struct id3 {
 
 int main (int argc, char *argv[])
 {
-	int deletetag=0, listtag=0, rfc822style=0, hastag=0, v11tag=0, i, r, exitcode=0;
+	int deletetag=0, listtag=0, rfc822style=0, hastag=0, notdir=0;
+	int v11tag=0, i, r, exitcode=0;
 	int matches, lettersmatched; /* genre lazy matching */
 	int newtitle=0, newartist=0, newalbum=0;
 	int newyear=0, newcomment=0, newgenre=0;
 	int newtrack=0;
 	struct id3 oldid3, newid3;
+	struct stat finfo;
 	FILE *fp;
 	
 	memset(&newid3, 0, 127);
@@ -167,7 +172,7 @@ int main (int argc, char *argv[])
 	}
 	
 	for (i=optind; i<argc; i++) {
-		hastag=1; /* innocent until proven guilty */
+		notdir=hastag=1; /* innocent until proven guilty */
 		if (newtitle || newartist || newalbum || newyear ||
 			newcomment || newtrack || newgenre || deletetag) {
 			fp = fopen(argv[i], "r+"); /* read/write */
@@ -180,6 +185,8 @@ int main (int argc, char *argv[])
 			exitcode=1;
 			continue;
 		}
+		fstat(fileno(fp), &finfo);
+		if (S_ISDIR(finfo.st_mode)) notdir=0;
 		
 		if (fseek(fp, -128, SEEK_END) < 0) {
 			/* problem rewinding */
@@ -240,7 +247,11 @@ int main (int argc, char *argv[])
         				else
         					printf("Comment: %-30.30s\n", oldid3.comment);
 				} else {
-					printf(" No ID3 tag.\n");
+					if (notdir) {
+						printf(" No ID3 tag.\n");
+					} else {
+						printf(" Directory.\n");
+					}
 				}
 			}
 		}
@@ -303,6 +314,13 @@ int main (int argc, char *argv[])
 			if (fwrite(&newid3, 128, 1, fp) < 0) exitcode=1;
 		}
 		fclose(fp);
+		
+		/* Reset tag flags so that we don't mistakenly assume the
+		 * next file also has a tag. (No need to reset hastag here,
+		 * because it is always set to 1 until a non-tag is found.
+		 */
+		oldid3.tag[0] = '\0';
+		
 	}
 	return exitcode;
 }
